import { CamelCase } from '.';
import { AssertString } from './String.types';
export type ToColonSingle<T extends string | number> = '' | `:${Exclude<T, ''>}`;
/**
 * Constructs a string type by concatenating three parts: `T`, `U`, and `V`,
 * separated by colons. The `U` and `V` parts are processed by the `ToColonSingle`
 * utility type before concatenation.
 *
 * @template T - The first part of the resulting string, must be a string or number.
 * @template U - The second part of the resulting string, must be a string or number.
 * @template V - The third part of the resulting string, must be a string or number, defaults to an empty string.
 */
export type ToAttr<T extends string | number, U extends string | number, V extends string | number = ''> = `${T}${ToColonSingle<U>}${ToColonSingle<V>}`;
/**
 * Constructs a type that represents a chain of attributes.
 *
 * @template T - The first attribute type, which can be a string or number.
 * @template U - The second attribute type, which can be a string or number.
 * @template V - The third attribute type, which can be a string or number. Defaults to an empty string.
 *
 * @remarks
 * This type combines the attributes using the `ToAttr` and `ToColonSingle` utility types.
 * It allows for a chain of attributes where the second attribute `U` is excluded if it is an empty string.
 *
 * @example
 * ```typescript
 * type Example1 = ToAttrChain<'attr1', 'attr2'>; // Results in ToAttr<'attr1', 'attr2', ''> | 'attr2' | ''
 * type Example2 = ToAttrChain<'attr1', 'attr2', 'attr3'>; // Results in ToAttr<'attr1', 'attr2', 'attr3'> | 'attr2:attr3' | 'attr3'
 * ```
 */
export type ToAttrChain<T extends string | number, U extends string | number, V extends string | number = ''> = ToAttr<T, U, V> | `${Exclude<U, ''>}${ToColonSingle<V>}` | V;
/** */
export type KebabToCamel<T extends Record<string, unknown>> = {
    [K in keyof T as CamelCase<AssertString<K>>]: T[K];
};
/** */
export type TypeFlatter<T extends Record<string, unknown>> = {
    [K in keyof T]: T[K];
};
/** ⚠️ This doesn't work yet: https://github.com/vuejs/core/issues/8286
 * @deprecated
 */
export type ToVueProps<T extends Record<string, unknown>> = {
    [K in keyof (T & {
        modelValue: T['model'] extends any ? unknown : never;
    }) as 'model' extends K ? never : CamelCase<AssertString<K>>]: 'modelValue' extends K ? T['model'] : T[K];
};
export type ToPayloadTuple<T> = NonNullable<T> extends void ? [] : [payload: NonNullable<T>];
/** ⚠️ This doesn't work yet: https://github.com/vuejs/core/issues/8286
 * @deprecated
 */
export type ToVueEmit<T extends Record<string, unknown>> = {
    -readonly [K in keyof Omit<T, 'change'>]-?: ToPayloadTuple<T[K]>;
} & (Required<T>['change'] extends undefined ? {} : {
    'update:modelValue': ToPayloadTuple<T['change']>;
});
